/*
 * Copyright (c) 2011, Noel Lemouel
 * All rights reserved.
 */

#include <p32xxxx.h>
#include <plib.h>

#include "uart.h"

#pragma config FCKSM = CSDCMD 		// Clk Sw Disabled, Fail Safe Clk Mon Disabled
#pragma config OSCIOFNC = OFF 		// CLKO (output on OSC0 pin) Disabled
#pragma config FSOSCEN = OFF 		// Secondary Oscillator Disabled
#pragma config CP = OFF 			// Code Protect Disabled
#pragma config BWP = OFF 			// Boot Flash write protect OFF
#pragma config PWP = OFF 			// Program Flash write protect OFF
#pragma config ICESEL = ICS_PGx2 	// ICE pins are shared with PGC2, PGD2
#pragma config DEBUG = OFF 			// ICE Background debugger = OFF

// Configuration Bit settings
//#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FWDTEN = OFF
#pragma config POSCMOD = HS, FNOSC = PRI, FPBDIV = DIV_1

// #define SYS_FREQ (4000000L)

/* ram base KSEG1 => 0xA0000000 */

const unsigned char ack[] = {'A', 'C', 'K', '\0'};
const unsigned char nack[] = {'N', 'A', 'C', 'K'};

void (*jump)(void) = 0x00;

int main(void)
{
   unsigned int i;
   unsigned int a = 0x00;
   unsigned int s = 0x00;
   unsigned char cmd[16];
   unsigned char* _p8 = 0x00;

// BMXCON = 0xXXXXXXXX;

   /* BMXDRMSZ = 16kB => 32MX320F128H */

   BMXDKPBA = 0x00000800;	/* RAM kernel data = 2kB (min size) */
   BMXDUDBA = 0x00004000;	/* RAM kernel program = 14kB */
   BMXDUPBA = 0x00004000;   /* RAM user data / program = 0 */

   uart1_init();

//   for(;;) uart_putc(uart_getc());

   /* command */

   // | D | L | a3 | a2 | a1 | a0 | - | s3 | s2 | s1 | s0 | - | - | - | - | DT |
   // DT = 'C' => code
   // DT = 'D' => data   

   for(i=0; i<16; i++)
   {
      cmd[i] = uart_getc();
   }

   /* parse */

   for(i=0; i<4; i++)
   {
      a <<= 8;
      a |= cmd[i+2];
   }

   _p8 = (unsigned char*)a;
   jump = (void*)_p8;

   for(i=0; i<4; i++)
   {
      s <<= 8;
      s |= cmd[i+7];
   }

   /* send ACK */

   for(i=0; i<4; i++)
   {
      uart_putc(ack[i]);
   }

   /* download */

   for(i=0; i<s; i++)
   {
      _p8[i] = uart_getc();
   }

   jump();

   return 0;
}
